
// (c) COPYRIGHT URI/MIT 1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//	reza		Reza Nekovei (reza@intcomm.net)

// DSP sub-class implementation for DSPByte,..DSPGrid.
// The files are patterned after the subcalssing examples 
// Test<type>.c,h files.
//
// ReZa 8/25/96

#include "config_dsp.h"

static char rcsid[]={"$Id: DSPArray.cc 13340 2006-03-03 16:10:13Z jimg $"};

#ifdef __GNUG__
#pragma implementation
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <string>
#include <iostream>

#include "dods-datatypes.h"
#include "cgi_util.h"
#include "Error.h"

#include "DSPArray.h"
#include "Dtype.h"

static char Msgt[255]; // Used by ErrMsgT

extern "C" int    OpenDsp(const char *filename);
extern "C" void   CloseDsp();
extern "C" void   cvtlng_(long *ilon, long *ilat, float *lon, float *lat);
extern "C" int    CheckImage(int *nline, int *npix);
extern "C" int    GetImageLine(char **buf, int LineNo);
extern "C" D_type VarType();

Array *
NewArray(const string &n, BaseType *v)
{
    return new DSPArray(n, v);
}

BaseType *
DSPArray::ptr_duplicate()
{
    return new DSPArray(*this);
}

DSPArray::DSPArray(const string &n, BaseType *v) : Array(n, v)
{
}

DSPArray::~DSPArray()
{
}

bool
DSPArray::read(const string &dataset)
{
    int start, stride, stop;
    int start_p, stride_p, stop_p;
    float lon, lat;
    int nline, npix; 
    dods_int32   *BufInt32;
    dods_float64 *BufFlt64;
    dods_byte    *dataBuf;

    if (read_p())  // Nothing to do
	return false;

    if (!OpenDsp(dataset.c_str())) {
	sprintf(Msgt, "DSPArray: Could not open file %s", dataset.c_str());
	ErrMsgT(Msgt);
	throw Error(cannot_read_file, 
		    "The Server could not open the dataset.");
    }

    Dim_iter p = dim_begin();
#if 0
    Pix p = first_dim();
#endif
    start = dimension_start(p,true);
    stride = dimension_stride(p, true);
    stop = dimension_stop(p, true); 

    if (!CheckImage(&nline, &npix)) {
	sprintf(Msgt, "DSPArray: Couldn't initialize image-input module");
	ErrMsgT(Msgt);
	throw Error(unknown_error, Msgt);
    } 
    // Get lat values from DSP
    if (name() == "lat") {
	if(start+stop+stride == 0){ //default
	    start = 0;
	    stride = 1;
	    stop = nline-1;
	}
	dods_float64 *flt64 = new dods_float64 [((stop - start)/stride) + 1];
	int count = 0;
	long ilon = 0;
	for (long ilat = start; ilat <= stop; ilat+=stride, count++) {
	    cvtlng_(&ilon, &ilat, &lon, &lat);
	    *(flt64+count) = (dods_float64) lat;
	}
	set_read_p(true);  
	val2buf((void *)flt64);      
	delete [] flt64; //clean up
    } 
    // Get lon values from DSP
    else if(name() == "lon") {
	if(start+stop+stride == 0){ //default
	    start = 0;
	    stride = 1;
	    stop = npix-1;
	}
	dods_float64 *flt64 = new dods_float64 [((stop - start)/stride) + 1];	
	int count = 0;
	long ilat = 0;
	for (long ilon = start; ilon <= stop; ilon+=stride, count++) {
	    cvtlng_(&ilon, &ilat, &lon, &lat);
	    *(flt64+count) = (dods_float64) lon;
	}
	set_read_p(true);  
	val2buf((void *)flt64);	
	delete [] flt64; //clean up
    }
    else {
	// Loop over number of lines (slower dimension) for actual data Array
	char    *buf; 
#if 0
	next_dim(p);
#endif
        ++p;
	start_p = dimension_start(p,true);
	stride_p = dimension_stride(p, true);
	stop_p = dimension_stop(p, true); 

	if(start+stop+stride == 0){ //default lines
	    start = 0;
	    stride = 1;
	    stop = nline-1;
	}
	if(start_p+stop_p+stride_p == 0){ //default pixels
	    start_p = 0;
	    stride_p = 1;
	    stop_p = npix-1;
	}

	int Len = (((stop-start)/stride)+1)*(((stop_p-start_p)/stride_p)+1);

	D_type DType = VarType();
	D_type ConvertFrom =  D_UNSPECIFIED;

	switch (DType) { // allocate memory and set type conversion
	  case D_SHORT:
	    if(sizeof(short) != sizeof(dods_int32))
		ConvertFrom = D_SHORT;
	    BufInt32 = new dods_int32 [Len]; 
	    break;

	  case D_LONG:
	    if(sizeof(Dlong) != sizeof(dods_int32))
		ConvertFrom = D_LONG;
	    BufInt32 = new dods_int32 [Len]; 
	    break;
	    
	  case D_FLOAT:
	    if(sizeof(float) != sizeof(dods_float64))
		ConvertFrom = D_FLOAT;
	    BufFlt64 = new dods_float64 [Len]; 	
	    break;

	  case D_DOUBLE:
	    if(sizeof(double) != sizeof(dods_float64))
		ConvertFrom = D_DOUBLE;
	    BufFlt64 = new dods_float64 [Len]; 	
	    break;
	  
	  default:	
	    dataBuf = new dods_byte [Len]; 
	    break;
	}

	int Tcount = 0;
	
	// typically no data conversion is needed.
	if(ConvertFrom == D_UNSPECIFIED) {
	    for (int iline = start; iline <= stop; iline+=stride) {	  
		if (!GetImageLine(&buf,iline)) {
		    sprintf(Msgt, "DSPArray: Error reading line %d from image",
			    iline);
		    ErrMsgT(Msgt);
		    throw Error(unknown_error, Msgt);
		}
		else{
		    for(int ipix = start_p; ipix <= stop_p; ipix+=stride_p){
			*(dataBuf+Tcount) = *(buf + ipix); 
			Tcount++;
		    }
		}
	    }
	    set_read_p(true);  
	    val2buf((void *)dataBuf);
	    delete [] dataBuf;
	}
	// Data needs type conversion
	else { 
	    for (int iline = start; iline <= stop; iline+=stride) {	  
		if (!GetImageLine(&buf,iline)) {
		    sprintf(Msgt, "DSPArray: Error reading line %d from image",
			    iline);
		    ErrMsgT(Msgt);
		    throw Error(unknown_error, Msgt);
		}
		else {
		    short  *bufsht = (short *) buf;
		    Dlong  *buflng = (Dlong *) buf;
		    float  *bufflt = (float *) buf;
		    double *bufdbl = (double *) buf;

		    for (int ipix = start_p; ipix <= stop_p; ipix+=stride_p) {
			switch (ConvertFrom) {
			  case D_SHORT:
			    *(BufInt32+Tcount) = (dods_int32) *(bufsht+ipix);  
			    break;

			  case D_LONG:
			    *(BufInt32+Tcount) = (dods_int32) *(buflng+ipix);  
			    break;
		
			  case D_FLOAT:
			    *(BufFlt64+Tcount) = (dods_float64) *(bufflt+ipix);
			    break;
		
			  case D_DOUBLE:
			    *(BufFlt64+Tcount) = (dods_float64) *(bufdbl+ipix);
			    break;
		  
			  default:	
			    ErrMsgT("DSPArray: Invalid data conversion");
			    throw InternalErr(__FILE__, __LINE__, 
			      "Server reports: Invalid data conversion.");
			}
		
			Tcount++;
		    }
		}
	    }

	    set_read_p(true);  

	    if((ConvertFrom == D_SHORT) || (ConvertFrom == D_LONG)){
		val2buf((void *)BufInt32);
		delete [] BufInt32;
	    }
	    else {
		val2buf((void *)BufFlt64);
		delete [] BufFlt64;
	    }

	}

    }

    CloseDsp();

    return false;
}

// $Log: DSPArray.cc,v $
// Revision 1.8  2004/02/19 16:12:33  jimg
// iostream.h --> iostream
//
// Revision 1.7  2003/02/10 21:19:02  jimg
// Merged changes from 3.2.4.
//
// Revision 1.6.4.2  2002/06/28 16:44:51  jimg
// Changes to the Connect class changed the set of headers included by other
// headers; I fixed the includes of these files to compensate.
//
// Revision 1.6.4.1  2002/01/30 20:00:55  jimg
// Formatting changes to read(...).
//
// Revision 1.6  2000/10/09 21:24:59  jimg
// Moved CVS Log entries to the end of each file.
// Changed the definition of the read method to match the dap library.
// Added exceptions to the read methods.
// Added code to catch exceptions in dsp_das, _dds and _dods.
//
// Revision 1.5  1999/04/30 21:32:45  jimg
// Merged no-gnu and release-2-17
//
// Revision 1.4.4.1  1999/02/10 06:28:56  brent
// convert to Standard C++ strings -- b^2
//
// Revision 1.4  1998/08/06 16:32:06  jimg
// Fixed misuse of the read(...) member function. Return true if more data
// is to be read, false is if not and error if an error is detected
//
// Revision 1.3  1996/11/15 18:22:12  jimg
// Fixed copyright statement.
// Added include of config header
//
// Revision 1.2  1996/10/15 16:14:28  reza
// Fixed the stride counter to get subsamples (not subsections).
//
// Revision 1.1  1996/09/18 22:41:54  reza
//  First release of DODS-DSP servers.
