
// (c) COPYRIGHT URI/MIT 1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//	reza		Reza Nekovei (reza@intcomm.net)

// DSP sub-class implementation for DSPByte,...DSPGrid.
// The files are patterned after the subcalssing examples 
// Test<type>.c,h files.
//
// ReZa 8/25/96

#include "config_dsp.h"

static char rcsid[] not_used ={"$Id: DSPGrid.cc 13340 2006-03-03 16:10:13Z jimg $"};

#include <string>

#include "InternalErr.h"
#include "DSPGrid.h"

Grid *
NewGrid(const string &n)
{
    return new DSPGrid(n);
}

// protected

BaseType *
DSPGrid::ptr_duplicate()
{
    return new DSPGrid(*this);
}

// public

DSPGrid::DSPGrid(const string &n) : Grid(n)
{
}

DSPGrid::~DSPGrid()
{
}

// read() should throw an Error or an InternalErr to indicate a problem with
// the read. The return value indicates that (true) more data needs to be
// read or (false) that all data was read.
bool
DSPGrid::read(const string &dataset)
{
    if (read_p()) // nothing to do
        return false;

    // read array elements
    if (array_var()->send_p())
	array_var()->read(dataset);

    // read maps elements
#if 0
    for (Pix p = first_map_var(); p; next_map_var(p)) {
	if (map_var(p)->send_p())
	    map_var(p)->read(dataset);
    }
#endif
    Map_iter p = map_begin();
    while (p != map_end()) {
        if ((*p)->send_p())
            (*p)->read(dataset);
        ++p;
    }

    set_read_p(true);

    return false;
}

// $Log: DSPGrid.cc,v $
// Revision 1.6  2003/02/10 21:19:02  jimg
// Merged changes from 3.2.4.
//
// Revision 1.5.4.1  2002/01/29 19:59:26  jimg
// I modified DSPGrid::read so that it checks the value of send_p() before
// reading the array and maps. This keeps the server from reading parts of a
// Grid that will never be sent to t he client.
// I also modified the method so that it always returns false. The old behavior
// of returning true or false to indicate whether more data needs to be read is
// deprecated. read() should always return false now. It should throw Error
// objects to indicate problems reading data. In the case of DSPGrid::read,
// these throws should be in the DSPArray::read implementations.
//
// Revision 1.5  2000/10/09 21:24:59  jimg
// Moved CVS Log entries to the end of each file.
// Changed the definition of the read method to match the dap library.
// Added exceptions to the read methods.
// Added code to catch exceptions in dsp_das, _dds and _dods.
//
// Revision 1.4  1999/04/30 21:32:46  jimg
// Merged no-gnu and release-2-17
//
// Revision 1.3.4.1  1999/02/10 06:28:57  brent
// convert to Standard C++ strings -- b^2
//
// Revision 1.3  1998/08/06 16:32:08  jimg
// Fixed misuse of the read(...) member function. Return true if more data
// is to be read, false is if not and error if an error is detected
//
// Revision 1.2  1996/11/15 18:22:17  jimg
// Fixed copyright statement.
// Added include of config header
//
// Revision 1.1  1996/09/18 22:42:02  reza
//  First release of DODS-DSP servers.
//
