
// -*- mode: c++; c-basic-offset:4 -*-

// This file is part of nc_handler, a data handler for the OPeNDAP data
// server. 

// Copyright (c) 2002,2003 OPeNDAP, Inc.
// Author: James Gallagher <jgallagher@opendap.org>
//
// This is free software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 2.1 of the License, or (at your
// option) any later version.
// 
// This software is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
// License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
 
#include <string>

#include "DSPByte.h"
#include "DSPInt16.h"
#include "DSPUInt16.h"
#include "DSPInt32.h"
#include "DSPUInt32.h"
#include "DSPFloat32.h"
#include "DSPFloat64.h"
#include "DSPStr.h"
#include "DSPUrl.h"
#include "DSPArray.h"
#include "DSPStructure.h"
#include "DSPSequence.h"
#include "DSPGrid.h"

#include "DSPTypeFactory.h"
#include <debug.h>

Byte *
DSPTypeFactory::NewByte(const string &n ) const 
{ 
    return new DSPByte(n);
}

Int16 *
DSPTypeFactory::NewInt16(const string &n ) const 
{ 
    return new DSPInt16(n); 
}

UInt16 *
DSPTypeFactory::NewUInt16(const string &n ) const 
{ 
    return new DSPUInt16(n);
}

Int32 *
DSPTypeFactory::NewInt32(const string &n ) const 
{ 
    DBG(cerr << "Inside DSPTypeFactory::NewInt32" << endl);
    return new DSPInt32(n);
}

UInt32 *
DSPTypeFactory::NewUInt32(const string &n ) const 
{ 
    return new DSPUInt32(n);
}

Float32 *
DSPTypeFactory::NewFloat32(const string &n ) const 
{ 
    return new DSPFloat32(n);
}

Float64 *
DSPTypeFactory::NewFloat64(const string &n ) const 
{ 
    return new DSPFloat64(n);
}

Str *
DSPTypeFactory::NewStr(const string &n ) const 
{ 
    return new DSPStr(n);
}

Url *
DSPTypeFactory::NewUrl(const string &n ) const 
{ 
    return new DSPUrl(n);
}

Array *
DSPTypeFactory::NewArray(const string &n , BaseType *v) const 
{ 
    return new DSPArray(n, v);
}

Structure *
DSPTypeFactory::NewStructure(const string &n ) const 
{ 
    return new DSPStructure(n);
}

Sequence *
DSPTypeFactory::NewSequence(const string &n ) const 
{ 
    DBG(cerr << "Inside DSPTypeFactory::NewSequence" << endl);
    return new DSPSequence(n);
}

Grid *
DSPTypeFactory::NewGrid(const string &n ) const 
{ 
    return new DSPGrid(n);
}
