/* 
  (c) COPYRIGHT URI/MIT 1996
  Please read the full copyright statement in the file COPYRIGH.  

  Authors:
      reza            Reza Nekovei (reza@intcomm.net)


  DODS server implementation for DSP images.
  ReZa 8/25/96 
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include        <stdarg.h>
#include        <time.h>

#include	<imgstdio.h>
#include	<dspmsgdef.h>
#include	<assdata.h>
#include	<projvars.h>
#include	<inghdrtir.h>
#include	<inghdrczc.h>
#include	<misc_header.h>
#include	<message.h>
#include        <imgerrno.h>
#include	<palette.h>

#include	"Dtype.h"

#define	DSP_BLOCKSIZE	512
#define	ReadBytes  Dsp_ReadBytes
#define	NPIX	   (ARRSIZ*4)	/* maximum number of bytes in a line */

static char pixbuf[NPIX];	/* pixel buffer */
IMGFILE *InImage;

extern void AddAtt();
extern void GlbAtt();
extern void StrAtt();
extern void GlbStrAtt();
extern int AddAttTab();
extern void SendError();
extern SUBIMAGEHEADER *Dsp_GetSubImageHeader();

/* This function could easily overflow the buffer 'buff.' 04/13/04 jhrg */
static void
ErrorExit(const char *myname, const char *fmt, ...)
{
    va_list alist;
    char buff[512];
    int l;
    time_t TimBin;
    char TimStr[26];
    char *host_or_addr = getenv("REMOTE_HOST") ? getenv("REMOTE_HOST") :
	getenv("REMOTE_ADDR");
    char *script = getenv("SCRIPT_NAME") ? getenv("SCRIPT_NAME") :
	"DODS server";

    va_start(alist, fmt);
#if 0
    myname = va_arg(alist, char *);
    fmt = va_arg(alist, char *);
#endif

    sprintf(buff, "%s:  ", myname);
    l = strlen(buff);
    vsprintf(buff + l, fmt, alist);
    va_end(alist);

    if (time(&TimBin) == (time_t) - 1)
	strcpy(TimStr, "time() error           ");
    else {
	strcpy(TimStr, ctime(&TimBin));
	TimStr[26 - 2] = '\0';
    }

    fprintf(stderr, "[%s] CGI: %s faild for %s reason: %s \n", TimStr,
	    script, host_or_addr, buff);

    /* this is where the error object is send to the server side */
    SendError(0, buff);
    exit(1);
}


int
OpenDsp(char *dataset)
{
    char varname[MAX_BAND_NAME];
    char fileName[256];
    char subName[MAX_ENTRY_NAME];
    char bandName[MAX_BAND_NAME];
    char title[MAX_TITLE];
    int irpcod;

#ifdef DEBUG
    IMGFILE *fout = 0;
    int irpcod = Dsp_OpenInputImage(dataset, &InImage, &fout);
#else				/* no need for information messages about subimage and bandname */
    Dsp_SplitImageName(dataset, fileName, subName, bandName, title);
    InImage = Dsp_ImageOpenQuiet(fileName, subName, bandName, "r");
    irpcod = ImageIoErrNo;
#endif

    if ((irpcod == DISPLAY_GOOD) | (irpcod == DISPLAY_NOCAL) |
	(irpcod == DISPLAY_NONAV))
	return 1;

    return 0;
}

void
CloseDsp()
{
    Dsp_ImageClose(&InImage);
}

D_type
VarType()
{
    int BandNumber = 0;
    int PixelType = Dsp_GetBandPixelType(&InImage, &BandNumber);
    int PixelSize = Dsp_GetBandPixelSize(&InImage, &BandNumber);
    D_type DataType;
    static char me[] = "dsp server: VarType,";	/* this routine's name */

    /* Determine type and size of data */
    switch (PixelType) {

    case DATA_TYPE_INT:{

	    switch (PixelSize) {

	    case 0:		/* default size */
	    case sizeof(char):
		DataType = D_BYTE;
		break;
	    case sizeof(short):
		DataType = D_SHORT;
		break;
	    case sizeof(Dlong):
		DataType = D_LONG;
		break;
	    default:
		ErrorExit(me,
			  "Integral input pixels have invalid number of bytes = %d",
			  PixelSize);
	    }
	    break;
	}

    case DATA_TYPE_REAL:{
	    switch (PixelSize) {

	    case 0:		/* default size */
	    case sizeof(float):
		DataType = D_FLOAT;
		break;
	    case sizeof(double):
		DataType = D_DOUBLE;
		break;
	    default:
		ErrorExit(me,
			  "Real input pixels have invalid number of bytes = %d",
			  PixelSize);
	    }
	    break;
	}

    default:
	ErrorExit(me, "Pixel type %d is neither integral nor real.",
		  PixelType);
    }

    return DataType;
}


/*   Define DODS Dataset Descriptor Structure object. */
int
DefineDDS()
{
    int ProjType = Dsp_ImageProj(&InImage);
    char VarName[MAX_BAND_NAME];
    char BandName[MAX_BAND_NAME];
    int BandNumber = 0;
    int DimSiz[2];
    static char me[] = "dsp server: DefineDDS,";	/* this routine's name */

    extern int MakeGrid();
    extern int MakeArray();

    DimSiz[0] = Dsp_GetSubImageSizeY(&InImage);
    DimSiz[1] = Dsp_GetSubImageSizeX(&InImage);

    if (strcpy(VarName, Dsp_GetBandName(&InImage, &BandNumber, BandName) ==
	       DISPLAY_GOOD
	       && strlen(BandName) > 0 ? BandName : "dsp_band_1") == NULL)
	ErrorExit(me, "Couldn't get band name `%s'");

    if (ProjType == PROJ_OLDREMAP || ProjType == PROJ_CYLINEQUIDIST
	|| ProjType == PROJ_CYLINEQUIRECT) {

	DBGc(me, "Making the grid type");
	if (!MakeGrid(VarName, VarType(), DimSiz))
	    ErrorExit(me, "Couldn't create grid");

    }
    else {

	DBGc(me, "Making the array type");
	if (!MakeArray(VarName, VarType(), DimSiz))
	    ErrorExit(me, "Couldn't create array");
    }

    Dsp_ImageClose(&InImage);
    return 1;
}


int
CheckImage(int *nline, int *npix)
{
    int status = 0;		/* return status = failure */
    int BandIndex = 0;		/* band index */
    int PixelType = Dsp_GetBandPixelType(&InImage, &BandIndex);
    int PixelSize = Dsp_GetBandPixelSize(&InImage, &BandIndex);
    static char me[] = "dsp server: CheckImage,";	/* this routine's name */

    if (PixelSize == 0) {
	if (PixelType == DATA_TYPE_INT) {
	    PixelSize = 1;
	}
	else if (PixelType == DATA_TYPE_REAL) {
	    PixelSize = 4;
	}
    }

    *nline = Dsp_GetSubImageSizeY(&InImage);
    *npix = Dsp_GetSubImageSizeX(&InImage);

    if (PixelType != DATA_TYPE_INT && PixelType != DATA_TYPE_REAL) {
	ErrorExit(me, "Pixel-type is not byte-aligned.  Can't handle %d",
		  PixelType);
    }
    else {
	if (PixelSize * (*npix) > NPIX) {
	    ErrorExit(me, "Input image line is too big.  Max = %d", NPIX);
	}
	else {
	    if ((*nline) <= 0) {
		ErrorExit(me, "Couldn't get number of lines in image");
	    }
	    else {
		status = 1;
	    }
	}
    }
    return status;
}

int
GetImageLine(char **buf, int LineNo)
{
    static char me[] = "dsp server: GetImageLine,";	/* this routine's name */

    LineNo++;			/* Origion here is one not zero */
    if (!Dsp_ImageRead(pixbuf, &LineNo, &InImage)) {

	ErrorExit(me, "Error reading line %d (origin 1) from input file",
		  LineNo);
	return 0;
    }

    *buf = pixbuf;

    return 1;
}


int
DefineDAS()
{
    static int status = 0;
    char varname[MAX_BAND_NAME];

    static char me[] = "dsp server: DefineDAS,";	/* this routine's name */

    DBGc(me, "Get the band name for pixel variable");
    if (!RasterName(varname)) {
	ErrorExit(me, "Couldn't get the band name");
    }
    else {
	DBGc(me, "Set mapping vectors");
	if (!DefineMaps()) {
	    ErrorExit(me, "Couldn't define Maps");
	}
	else {
	    DBGc(me, "Converting sub-image header");
	    if (!ConvertSubHead()) {
		ErrorExit(me, "Couldn't convert sub-image header");
	    }
	    else {
		DBGc(me, "Converting band information");
		if (!ConvertBandInfo(InImage, varname)) {
		    ErrorExit(me, "Couldn't convert band information");
		}
		else {
		    DBGc(me, "Converting associated blocks");
		    if (!ConvertAssocBlocks(varname)) {
			ErrorExit(me,
				  "Couldn't convert associated blocks");
		    }
		    else {
			status = 1;
		    }
		}
	    }
	}
    }
    Dsp_ImageClose(&InImage);
    return status;
}


/*
 *	Define the mapping attibutte variables.
 */
int
DefineMaps()
{
    int ProjType = Dsp_ImageProj(&InImage);

    if (ProjType == PROJ_OLDREMAP
	|| ProjType == PROJ_CYLINEQUIDIST
	|| ProjType == PROJ_CYLINEQUIRECT) {

	DBGc(me, "Defining latitude attributes");
	StrAtt("lat", "units", "degrees_north");
	StrAtt("lat", "long_name", "latitude");

	DBGc(me, "Defining longitude attributes");
	StrAtt("lon", "units", "degrees_east");
	StrAtt("lon", "long_name", "longitude");
    }

    return 1;
}

/*
 *     Get the grid/array variable name.
 */
int
RasterName(char *varname)
{
    char BandName[MAX_BAND_NAME];	/* input image band name */
    int BandNumber = 0;		/* input image band number */
    static char me[] = "dsp server: RasterName,";

    if (strcpy(varname, Dsp_GetBandName(&InImage, &BandNumber, BandName) ==
	       DISPLAY_GOOD
	       && strlen(BandName) > 0 ? BandName : "dsp_band_1") == NULL)
	ErrorExit(me, "Couldn't get band name `%s'");

    return 1;
}

/*  Parts of the following functions were taken largely from mia2cdf
 *   program. Copyright 1988-1992 by Rosenstiel School of Marine and
 *   Atmospheric Science,  University of Miami, Miami, Florida.
 */

/*	Convert the subimage header. */
int
ConvertSubHead()
{
    char one = 1;

#	define	SUBH(x)		(InImage->SubImageHeader->x)


    GlbAtt("dsp_SubImageId", D_SHORT, 1, &SUBH(SubImageId));
    GlbStrAtt("dsp_SubImageName", SUBH(SubImageName));
    GlbAtt("dsp_ModificationDate", D_LONG, 1, &SUBH(ModificationDate));
    GlbAtt("dsp_ModificationTime", D_LONG, 1, &SUBH(ModificationTime));
    GlbAtt("dsp_SubImageFlag", D_LONG, 1, &SUBH(SubImageFlag));
    GlbStrAtt("dsp_SubImageTitle", SUBH(SubImageTitle));
    GlbAtt("dsp_StartDate", D_LONG, 1, &SUBH(StartDate));
    GlbAtt("dsp_StartTime", D_FLOAT, 1, &SUBH(StartTime));
    GlbAtt("dsp_SizeX", D_LONG, 1, &SUBH(SizeX));
    GlbAtt("dsp_SizeY", D_LONG, 1, &SUBH(SizeY));
    GlbAtt("dsp_OffsetX", D_LONG, 1, &SUBH(OffsetX));
    GlbAtt("dsp_RecordLength", D_LONG, 1, &SUBH(RecordLength));
    GlbAtt("dsp_DataOrganization", D_BYTE, 1, &SUBH(DataOrganization));
    GlbAtt("dsp_NumberOfBands", D_BYTE, 1, &one);

    return 1;
}

/*
 *	Convert the band information.
 */
int
ConvertBandInfo(IMGFILE * ifd, char *varname)
{
    char pixelType;
    char pixelSize;
    int bandFlag;
    short nBits;
    int bandNumber = 0;		/* Current band */
    long bandLineSize;
    BANDDESC *band = &ifd->SubImageHeader->BandDesc[ifd->BandNumber];

    pixelType = Dsp_GetBandPixelType(&ifd, &bandNumber);
    pixelSize = Dsp_GetBandPixelSize(&ifd, &bandNumber);

    Dsp_GetBandPixelFlags(&ifd, &bandNumber, &bandFlag);

    nBits = Dsp_GetBandBitSize(&ifd, &bandNumber);
    bandLineSize = band->BandLineSize;

    AddAtt(varname, "dsp_PixelType", D_BYTE, 1, &pixelType);
    AddAtt(varname, "dsp_PixelSize", D_BYTE, 1, &pixelSize);
    AddAtt(varname, "dsp_Flag", D_SHORT, 1, &bandFlag);
    AddAtt(varname, "dsp_nBits", D_SHORT, 1, &nBits);
    AddAtt(varname, "dsp_LineSize", D_LONG, 1, &bandLineSize);

    return 1;
}


/*
 *	Convert the associated blocks in the input DSP image to the output
 *	netCDF object.
 */
int
ConvertAssocBlocks(char *varname)
{
    int status = 1;		/* returned status = success */
    int block =
	((SUBIMAGEHEADER *) Dsp_GetSubImageHeader(&InImage))->
	AssocDataPointer;
    int StartBlock, j;
    ASSOC_DIR *a = InImage->AssDirEntry;	/* associated-block directory */
    ASSOC_ENTRY *ae;
    static char me[] = "dsp server: ConvertAssocBlocks,";	/* this routine's name */

    for (StartBlock = block; status;) {
	for (j = 0; status && j < MAX_ASS_ENTRIES; j++) {
	    ae = &(a->AssEntry[j]);
	    if (ae->AssocBlockPtr != 0 &&
		((ae->AssocFlag & ASS_FLAG_DELETED) == 0)) {

		switch (ae->AssocDataType) {

		case ASS_NAVIGATION:
		    status = ConvertNavBlock();
		    break;
		case ASS_NAVIGATION_HEIGHT:
		    status = ConvertNavHgtBlock();
		    break;
		case ASS_PAL_DIR:
		    status = ConvertPalDirBlock(ae);
		    break;
		case ASS_CALIBRATION:
		    status = ConvertCalBlock(ae, varname);
		    break;
		case ASS_BAND_DIR:	/* already done */
		    break;
		case ASS_INGEST_HEADER:
		    status = ConvertIngestHeader(ae);
		    break;
		case ASS_CZCS_SUBCOM:
		    status = ConvertCzcsSubcom(ae);
		    break;
		case ASS_AUDIT:
		    status = ConvertAuditRecords(ae);
		    break;
		case ASS_DB_CONTROL:
		    status = ConvertDbControl(ae);
		    break;
		case ASS_PALETTE_1:
		case ASS_PALETTE_2:
		case ASS_PALETTE_3:
		case ASS_PALETTE_4:
		case ASS_PALETTE_5:
		case ASS_PALETTE_6:
		case ASS_PALETTE_7:
		case ASS_PALETTE_8:
		    status = ConvertPalette(ae);
		    break;
		default:
		    fprintf(stderr, "%s: Unknown associated data type: %d",
			    me, ae->AssocDataType);
		}		/* associated-block-type switch */
	    }			/* if valid associated block */
	}			/* associated-block, directory- */
	/* entry loop */
	/*
	 *      Read next set of asssociated-block, directory-entries.
	 */
	if (status) {
	    int nbytes = sizeof(ASSOC_DIR);	/* amount to read in bytes */

	    if ((block = a->NextAssocBlock) == StartBlock)
		break;		/* wrapped around => all done */
	    if (!ReadBytes
		(&InImage->FileDescriptor, (char *) a, &block, &nbytes)) {
		ErrorExit(me, "Couldn't read associated data block");
		status = 0;
	    }
	}
    }

    return status;
}


/*
 *	Convert the associated navigation block.
 */
int
ConvertNavBlock()
{
    int status = 1;		/* return status = success */
    static int initIt = 1;

    static AttEntry_t FixedNav[] = {
	{"dsp_nav_xsatid", D_CHAR, 4, odata.xsatid},
	{"dsp_nav_xsatty", D_BYTE, 1, (char *) &odata.xsatty},
	{"dsp_nav_xproty", D_BYTE, 1, (char *) &odata.xproty},
	{"dsp_nav_xmapsl", D_BYTE, 1, (char *) &odata.xmapsl},
	{"dsp_nav_xtmpch", D_BYTE, 1, (char *) &odata.xtmpch},
	{"dsp_nav_ximgdy", D_FLOAT, 1, (char *) &odata.ximgdy},
	{"dsp_nav_ximgtm", D_FLOAT, 1, (char *) &odata.ximgtm},
	{"dsp_nav_xorbit", D_FLOAT, 1, (char *) &odata.xorbit},
	{"dsp_nav_ximgcv", D_FLOAT, 8, (char *) odata.ximgcv},
	{NULL}
    };
    static char me[] = "dsp server: ConvertNavBlock,";
    /* this routine's name */

#define	GOESMC	((NAV_GOES_MC_t*)odata.zprjvr.proj01)
    static AttEntry_t GoesMcNav[] = {
	{"dsp_nav_goesmc_navday", D_LONG, 1, 0},
	{"dsp_nav_goesmc_lintot", D_LONG, 1, 0},
	{"dsp_nav_goesmc_deglin", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_ieltot", D_LONG, 1, 0},
	{"dsp_nav_goesmc_degele", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_spinra", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_ietimy", D_LONG, 1, 0},
	{"dsp_nav_goesmc_ietimh", D_LONG, 1, 0},
	{"dsp_nav_goesmc_semima", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_oeccen", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_orbinc", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_perhel", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_asnode", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_declin", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_rascen", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_piclin", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_prerat", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_predir", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_pitch", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_yaw", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_roll", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_betain", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_betdot", D_FLOAT, 1, 0},
	{"dsp_nav_goesmc_pictim", D_FLOAT, 1, 0},
	{NULL}
    };

#define	SAT	((NAV_SAT_t*)odata.zprjvr.proj01)
    static AttEntry_t RsmasSatNav[] = {
	{"dsp_nav_sat_lon0", D_FLOAT, 1, 0},
	{"dsp_nav_sat_lat0", D_FLOAT, 1, 0},
	{"dsp_nav_sat_scnlen", D_FLOAT, 1, 0},
	{"dsp_nav_sat_xmul", D_FLOAT, 1, 0},
	{"dsp_nav_sat_ymul", D_FLOAT, 1, 0},
	{"dsp_nav_sat_satper", D_FLOAT, 1, 0},
	{"dsp_nav_sat_earper", D_FLOAT, 1, 0},
	{"dsp_nav_sat_rpm", D_FLOAT, 1, 0},
	{"dsp_nav_sat_sample", D_FLOAT, 1, 0},
	{"dsp_nav_sat_scalht", D_FLOAT, 1, 0},
	{"dsp_nav_sat_lat0sn", D_FLOAT, 1, 0},
	{"dsp_nav_sat_lon0sn", D_FLOAT, 1, 0},
	{"dsp_nav_sat_radear", D_FLOAT, 1, 0},
	{"dsp_nav_sat_sgn", D_FLOAT, 1, 0},
	{"dsp_nav_sat_scan0", D_FLOAT, 1, 0},
	{"dsp_nav_sat_elt0", D_FLOAT, 1, 0},
	{"dsp_nav_sat_magx", D_FLOAT, 1, 0},
	{"dsp_nav_sat_magy", D_FLOAT, 1, 0},
	{"dsp_nav_sat_duble", D_FLOAT, 1, 0},
	{"dsp_nav_sat_aspect", D_FLOAT, 1, 0},
	{"dsp_nav_sat_an", D_FLOAT, 1, 0},
	{"dsp_nav_sat_bn", D_FLOAT, 1, 0},
	{"dsp_nav_sat_satrol", D_FLOAT, 1, 0},
	{"dsp_nav_sat_sattil", D_FLOAT, 1, 0},
	{"dsp_nav_sat_satday", D_FLOAT, 1, 0},
	{"dsp_nav_sat_sattim", D_FLOAT, 1, 0},
	{"dsp_nav_sat_satyaw", D_FLOAT, 1, 0},
	{"dsp_nav_sat_ascdot", D_FLOAT, 1, 0},
	{"dsp_nav_sat_yawl", D_FLOAT, 1, 0},
	{"dsp_nav_sat_norbit", D_FLOAT, 1, 0},
	{"dsp_nav_sat_verson", D_FLOAT, 1, 0},
	{"dsp_nav_sat_hrpvrc", D_FLOAT, 1, 0},
	{"dsp_nav_sat_colbas", D_FLOAT, 1, 0},
	{"dsp_nav_sat_rowbas", D_FLOAT, 1, 0},
	{"dsp_nav_sat_rflipx", D_FLOAT, 1, 0},
	{"dsp_nav_sat_rflipy", D_FLOAT, 1, 0},
	{"dsp_nav_sat_tilt", D_FLOAT, 1, 0},
	{"dsp_nav_sat_roll", D_FLOAT, 1, 0},
	{"dsp_nav_sat_timrev", D_FLOAT, 1, 0},
	{"dsp_nav_sat_geomsw", D_FLOAT, 1, 0},
	{"dsp_nav_sat_scncnt", D_FLOAT, 1, 0},
	{"dsp_nav_sat_asnode", D_FLOAT, 1, 0},
	{"dsp_nav_sat_magx2", D_FLOAT, 1, 0},
	{"dsp_nav_sat_magy2", D_FLOAT, 1, 0},
	{"dsp_nav_sat_scan02", D_FLOAT, 1, 0},
	{"dsp_nav_sat_elt02", D_FLOAT, 1, 0},
	{"dsp_nav_sat_sflipx", D_FLOAT, 1, 0},
	{"dsp_nav_sat_perdot", D_FLOAT, 1, 0},
	{NULL}
    };

#define	EARTH	((NAV_EARTH_t*)odata.zprjvr.proj01)
    static AttEntry_t RsmasEarthNav1[] = {
	{"dsp_nav_earth_linoff", D_FLOAT, 1, 0},
	{"dsp_nav_earth_pixoff", D_FLOAT, 1, 0},
	{"dsp_nav_earth_scnstr", D_FLOAT, 1, 0},
	{"dsp_nav_earth_scnstp", D_FLOAT, 1, 0},
	{"dsp_nav_earth_pixstr", D_FLOAT, 1, 0},
	{"dsp_nav_earth_pixstp", D_FLOAT, 1, 0},
	{"dsp_nav_earth_latorg", D_FLOAT, 1, 0},
	{"dsp_nav_earth_lonorg", D_FLOAT, 1, 0},
	{"dsp_nav_earth_orgrot", D_FLOAT, 1, 0},
	{"dsp_nav_earth_lattop", D_FLOAT, 1, 0},
	{"dsp_nav_earth_latbot", D_FLOAT, 1, 0},
	{"dsp_nav_earth_latcen", D_FLOAT, 1, 0},
	{"dsp_nav_earth_loncen", D_FLOAT, 1, 0},
	{"dsp_nav_earth_height", D_FLOAT, 1, 0},
	{"dsp_nav_earth_width", D_FLOAT, 1, 0},
	{"dsp_nav_earth_level", D_FLOAT, 1, 0},
	{"dsp_nav_earth_xspace", D_FLOAT, 1, 0},
	{"dsp_nav_earth_yspace", D_FLOAT, 1, 0},
	{"dsp_nav_earth_rev", D_CHAR, 4, 0},
	{"dsp_nav_earth_dflag", D_FLOAT, 1, 0},
	{NULL}
    };

    static AttEntry_t RsmasEarthNav2[] = {
	{"dsp_nav_earth_cenlon", D_DOUBLE, 1, 0},
	{"dsp_nav_earth_cenlat", D_DOUBLE, 1, 0},
	{"dsp_nav_earth_edglon", D_DOUBLE, 1, 0},
	{"dsp_nav_earth_edglat", D_DOUBLE, 1, 0},
	{NULL}
    };

    static AttEntry_t RsmasEarthNav3[] = {
	{"dsp_nav_earth_toplat", D_FLOAT, 1, 0},
	{"dsp_nav_earth_botlat", D_FLOAT, 1, 0},
	{"dsp_nav_earth_leflon", D_FLOAT, 1, 0},
	{"dsp_nav_earth_ritlon", D_FLOAT, 1, 0},
	{NULL}
    };

    static AttEntry_t RsmasEarthNav4[] = {
	{"dsp_nav_earth_numpix", D_FLOAT, 1, 0},
	{"dsp_nav_earth_numras", D_FLOAT, 1, 0},
	{"dsp_nav_earth_magxx", D_FLOAT, 1, 0},
	{"dsp_nav_earth_magyy", D_FLOAT, 1, 0},
	{NULL}
    };

    if (initIt) {
	AttEntry_t *g;

	g = GoesMcNav;
	g->value = (char *) &GOESMC->navday;
	g++;
	g->value = (char *) &GOESMC->lintot;
	g++;
	g->value = (char *) &GOESMC->deglin;
	g++;
	g->value = (char *) &GOESMC->ieltot;
	g++;
	g->value = (char *) &GOESMC->degele;
	g++;
	g->value = (char *) &GOESMC->spinra;
	g++;
	g->value = (char *) &GOESMC->ietimy;
	g++;
	g->value = (char *) &GOESMC->ietimh;
	g++;
	g->value = (char *) &GOESMC->semima;
	g++;
	g->value = (char *) &GOESMC->oeccen;
	g++;
	g->value = (char *) &GOESMC->orbinc;
	g++;
	g->value = (char *) &GOESMC->perhel;
	g++;
	g->value = (char *) &GOESMC->asnode;
	g++;
	g->value = (char *) &GOESMC->declin;
	g++;
	g->value = (char *) &GOESMC->rascen;
	g++;
	g->value = (char *) &GOESMC->piclin;
	g++;
	g->value = (char *) &GOESMC->prerat;
	g++;
	g->value = (char *) &GOESMC->predir;
	g++;
	g->value = (char *) &GOESMC->pitch;
	g++;
	g->value = (char *) &GOESMC->yaw;
	g++;
	g->value = (char *) &GOESMC->roll;
	g++;
	g->value = (char *) &GOESMC->betain;
	g++;
	g->value = (char *) &GOESMC->betdot;
	g++;
	g->value = (char *) &GOESMC->pictim;
	g++;

	g = RsmasSatNav;
	g->value = (char *) &SAT->lon0;
	g++;
	g->value = (char *) &SAT->lat0;
	g++;
	g->value = (char *) &SAT->scnlen;
	g++;
	g->value = (char *) &SAT->xmul;
	g++;
	g->value = (char *) &SAT->ymul;
	g++;
	g->value = (char *) &SAT->satper;
	g++;
	g->value = (char *) &SAT->earper;
	g++;
	g->value = (char *) &SAT->rpm;
	g++;
	g->value = (char *) &SAT->sample;
	g++;
	g->value = (char *) &SAT->scalht;
	g++;
	g->value = (char *) &SAT->lat0sn;
	g++;
	g->value = (char *) &SAT->lon0sn;
	g++;
	g->value = (char *) &SAT->radear;
	g++;
	g->value = (char *) &SAT->sgn;
	g++;
	g->value = (char *) &SAT->scan0;
	g++;
	g->value = (char *) &SAT->elt0;
	g++;
	g->value = (char *) &SAT->magx;
	g++;
	g->value = (char *) &SAT->magy;
	g++;
	g->value = (char *) &SAT->duble;
	g++;
	g->value = (char *) &SAT->aspect;
	g++;
	g->value = (char *) &SAT->an;
	g++;
	g->value = (char *) &SAT->bn;
	g++;
	g->value = (char *) &SAT->satrol;
	g++;
	g->value = (char *) &SAT->sattil;
	g++;
	g->value = (char *) &SAT->satday;
	g++;
	g->value = (char *) &SAT->sattim;
	g++;
	g->value = (char *) &SAT->satyaw;
	g++;
	g->value = (char *) &SAT->ascdot;
	g++;
	g->value = (char *) &SAT->yawl;
	g++;
	g->value = (char *) &SAT->norbit;
	g++;
	g->value = (char *) &SAT->verson;
	g++;
	g->value = (char *) &SAT->hrpvrc;
	g++;
	g->value = (char *) &SAT->colbas;
	g++;
	g->value = (char *) &SAT->rowbas;
	g++;
	g->value = (char *) &SAT->rflipx;
	g++;
	g->value = (char *) &SAT->rflipy;
	g++;
	g->value = (char *) &SAT->tilt;
	g++;
	g->value = (char *) &SAT->roll;
	g++;
	g->value = (char *) &SAT->timrev;
	g++;
	g->value = (char *) &SAT->geomsw;
	g++;
	g->value = (char *) &SAT->scncnt;
	g++;
	g->value = (char *) &SAT->asnode;
	g++;
	g->value = (char *) &SAT->magx2;
	g++;
	g->value = (char *) &SAT->magy2;
	g++;
	g->value = (char *) &SAT->scan02;
	g++;
	g->value = (char *) &SAT->elt02;
	g++;
	g->value = (char *) &SAT->sflipx;
	g++;
	g->value = (char *) &SAT->perdot;
	g++;

	g = RsmasEarthNav1;
	g->value = (char *) &EARTH->linoff;
	g++;
	g->value = (char *) &EARTH->pixoff;
	g++;
	g->value = (char *) &EARTH->scnstr;
	g++;
	g->value = (char *) &EARTH->scnstp;
	g++;
	g->value = (char *) &EARTH->pixstr;
	g++;
	g->value = (char *) &EARTH->pixstp;
	g++;
	g->value = (char *) &EARTH->latorg;
	g++;
	g->value = (char *) &EARTH->lonorg;
	g++;
	g->value = (char *) &EARTH->orgrot;
	g++;
	g->value = (char *) &EARTH->lattop;
	g++;
	g->value = (char *) &EARTH->latbot;
	g++;
	g->value = (char *) &EARTH->latcen;
	g++;
	g->value = (char *) &EARTH->loncen;
	g++;
	g->value = (char *) &EARTH->height;
	g++;
	g->value = (char *) &EARTH->width;
	g++;
	g->value = (char *) &EARTH->level;
	g++;
	g->value = (char *) &EARTH->xspace;
	g++;
	g->value = (char *) &EARTH->yspace;
	g++;
	g->value = (char *) EARTH->rev;
	g++;
	g->value = (char *) &EARTH->dflag;
	g++;

	g = RsmasEarthNav2;
	g->value = (char *) &EARTH->cenlon;
	g++;
	g->value = (char *) &EARTH->cenlat;
	g++;
	g->value = (char *) &EARTH->edglon;
	g++;
	g->value = (char *) &EARTH->edglat;
	g++;

	g = RsmasEarthNav3;
	g->value = (char *) &EARTH->OldRemap.toplat;
	g++;
	g->value = (char *) &EARTH->OldRemap.botlat;
	g++;
	g->value = (char *) &EARTH->OldRemap.leflon;
	g++;
	g->value = (char *) &EARTH->OldRemap.ritlon;
	g++;

	g = RsmasEarthNav4;
	g->value = (char *) &EARTH->numpix;
	g++;
	g->value = (char *) &EARTH->numras;
	g++;
	g->value = (char *) &EARTH->magxx;
	g++;
	g->value = (char *) &EARTH->magyy;
	g++;

	initIt = 0;
    }
    if (!AddAttTab("NC_GLOBAL", FixedNav)) {
	ErrorExit(me, "Couldn't convert fixed-navigation attributes");
	status = 0;
    }
    else {
	switch (odata.xsatty) {

	case 1:{		/* GOES McIDAS */
		if (!AddAttTab("NC_GLOBAL", GoesMcNav)) {
		    ErrorExit(me,
			      "Couldn't convert GOES McIDAS attributes");
		    status = 0;
		}
		else {
		    status = 1;
		}
		break;
	    }

	case 2:		/* GOES NESDIS */
	    ErrorExit(me, "Can't convert GOES NESDIS navigation yet");
	    status = 0;
	    break;

	case 4:		/* GOES CSU */
	    ErrorExit(me, "Can't convert GOES CSU navigation yet");
	    status = 0;
	    break;

	case 5:		/* RSMAS */
	    if (odata.xproty == 1) {
		if (!AddAttTab("NC_GLOBAL", RsmasSatNav)) {
		    ErrorExit(me,
			      "Couldn't convert RSMAS satellite-navigation attributes");
		    status = 0;
		}
		else {
		    status = 1;
		}
	    }			/* endif satellite system */
	    else if (odata.xproty > 1) {	/* if remapped system */
		if (!AddAttTab("NC_GLOBAL", RsmasEarthNav1)) {
		    ErrorExit(me,
			      "Couldn't convert main RSMAS Earth-navigation");
		    status = 0;
		}
		if (status && EARTH->dflag == 1.0) {
		    if (!AddAttTab("NC_GLOBAL", RsmasEarthNav2)) {
			ErrorExit(me,
				  "Couldn't convert double-precision RSMAS Earth-navigation");
			status = 0;
		    }
		}
		if (status && odata.xproty == 2) {
		    if (!AddAttTab("NC_GLOBAL", RsmasEarthNav3)) {
			ErrorExit(me,
				  "Couldn't convert old-style remap RSMAS Earth-navigation");
			status = 0;
		    }
		}
		if (status) {
		    if (!AddAttTab("NC_GLOBAL", RsmasEarthNav4)) {
			ErrorExit(me,
				  "Couldn't convert RSMAS Earth-navigation sizing");
			status = 0;
		    }
		}
	    }			/* endif remapped system */
	    else {		/* if odata.xproty < 0 */
		ErrorExit(me, "Unknown XPROTY = %d", odata.xproty);
		status = 0;
	    }
	    break;

	default:
	    ErrorExit(me, "Invalid XSATTY = %d", odata.xsatty);
	    status = 0;
	}			/* end odata.xsatty switch */
    }				/* if fixed-nav saved OK */

    return status;
}


/*
 *	Convert a DSP navigation-height block.
 */
int
ConvertNavHgtBlock()
{
    static char me[] = "dsp server: ConvertNavHgtBlock,";	/* this routine's name */

    static AttEntry_t AltNav[] = {
	{"dsp_hgt_llnval", D_LONG, 1, (char *) &sdata.llnval},
	{"dsp_hgt_lltime", D_LONG, 1, (char *) &sdata.lltime},
	{"dsp_hgt_llvect", D_FLOAT,
	 sizeof(sdata.llvect) / sizeof(float), (char *) sdata.llvect},
	    {NULL}
    };

    if (!AddAttTab("NC_GLOBAL", AltNav))
	ErrorExit(me, "Couldn't convert RSMAS navigation-height block");

    return 1;
}


/*
 *	Convert a palette directory block.
 */
int
ConvertPalDirBlock(ASSOC_ENTRY * ae)
{
    int status = 0;		/* return status = failure */
    int block = ae->AssocBlockPtr;	/* which block */
    int nbytes = ae->AssocTotalSize;
    register int i;
    PALDIR palDir;		/* palette directory */
    static char me[] = "dsp server: ConvertPalDirBlock,";	/* this routine's name */

    if (!ReadBytes
	(&InImage->FileDescriptor, (char *) &palDir, &block, &nbytes)) {
	ErrorExit(me, "Couldn't read palette-directory block");
    }
    else {

	for (i = 0; i < MAX_PALETTES; ++i) {
	    PALENTRY *pe = &palDir.PalEntry[i];

	    if (pe->palFlag & PAL_SLOT_BUSY) {
		char name[DSP_BLOCKSIZE];

		(void) sprintf(name, "dsp_pal_title_%d", i + 1);

		GlbStrAtt(name, pe->palTitle);
	    }
	}
    }

    status = i == MAX_PALETTES;

    return status;
}


/*
 *	Convert a DSP calibration block.
 */
int
ConvertCalBlock(ASSOC_ENTRY * ae, char *varname)
{
    int status = 1;		/* return status */
    int WhichBand = 0;		/* which band */
    CAL_BLOCK CalBlock;
    static char me[] = "dsp server: ConvertCalBlock,";

    if (Dsp_ReadCalibration(&InImage, &WhichBand, &CalBlock) !=
	DISPLAY_GOOD) {
	ErrorExit(me, "Couldn't get calibration block");
	status = 0;
    }
    else {
	CAL_BLOCK *calp = &CalBlock;
	long type = calp->eqnNumber;
	float slope;
	float intercept;

	/*
	 *      Set calibration attributes for current band.
	 */
	StrAtt(varname, "dsp_cal_name", calp->name);
	StrAtt(varname, "units", calp->units);
	AddAtt(varname, "dsp_cal_eqnNumber", D_SHORT, 1, &calp->eqnNumber);
	AddAtt(varname, "dsp_cal_CoeffsLength", D_SHORT, 1,
	       &calp->CoeffsLength);
	AddAtt(varname, "dsp_cal_coeffs", D_FLOAT,
	       calp->CoeffsLength / sizeof(float), calp->ucal.coeffs);

	if (type == EQN_LINEAR || type == EQN_LOG) {
	    slope = calp->ucal.eqn2n3.slope;
	    intercept = calp->ucal.eqn2n3.intercept;
	}
	else if (type == EQN_SATELLITE) {
#if 0
	    slope = (1 << *(int *) calp->coeffs) * calp->coeffs[2];
	    intercept = calp->coeffs[3] -
		calp->coeffs[2] ** (int *) (calp->coeffs + 1);
#endif
	    slope = (1 << calp->ucal.eqn1.scale) * calp->ucal.eqn1.rslope;
	    intercept = calp->ucal.eqn1.rintcp -
		calp->ucal.eqn1.rslope * calp->ucal.eqn1.bias;
	}
	else {
	    ErrorExit(me, "Unknown calibration type %d", type);
	    status = 0;
	}

	AddAtt(varname, "scale_factor", D_FLOAT, 1, &slope);
	AddAtt(varname, "add_off", D_FLOAT, 1, &intercept);

    }				/* if calibration read */

    return status;
}


/*
 *	Convert an ingest header.
 */
int
ConvertIngestHeader(ASSOC_ENTRY * ae)
{
    int status = 0;
    int nbytes = ae->AssocTotalSize;	/* amount to read */
    int block = ae->AssocBlockPtr;	/* which block */
    int nchar = 2;		/* number to compare */
    TIRO_INGEST_HDR header;
    static char czcsid[] = "CZCS";
    static char tiroid[] = "TIRO";
    static char noaaid[] = "NOAA";
    static char goesid[] = "GOES";
    static char me[] = "dsp server: ConvertIngestHeader,";

    if (!ReadBytes
	(&InImage->FileDescriptor, (char *) &header, &block, &nbytes)) {
	ErrorExit(me, "Couldn't read ingest header");
    }
    else {
	if (strncmp(header.ourid, czcsid, nchar) == 0) {
	    if (!ConvertCzcs(&header)) {
		ErrorExit(me, "Couldn't convert CZCS ingest-block");
	    }
	    else {
		status = 1;
	    }
	}
	else if (strncmp(header.ourid, tiroid, nchar) == 0) {
	    if (!ConvertTiros(&header)) {
		ErrorExit(me, "Couldn't convert TIROS ingest-block");
	    }
	    else {
		status = 1;
	    }
	}
	else if (strncmp(header.ourid, noaaid, nchar) == 0) {
	    if (!ConvertTiros(&header)) {
		ErrorExit(me, "Couldn't convert NOAA ingest-block");
	    }
	    else {
		status = 1;
	    }
	}
	else if (strncmp(header.ourid, goesid, nchar) == 0) {
	    fprintf(stderr, "%s: Don't know about GOES ingest-block", me);
	    status = 1;
	}
	else {
	    ErrorExit(me, "Unknown ingest header type '%4.4s'",
		      header.ourid);
	}
    }

    return status;
}


/*
 *	Convert a CZCS ingest-block.
 */
int
ConvertCzcs(CZCS_INGEST_HDR * b)
{
    int status = 1;

    GlbAtt("dsp_ing_czcs_ourid", D_CHAR, 8, b->ourid);
    GlbAtt("dsp_ing_czcs_tapseq", D_LONG, 1, &b->tapseq);
    GlbAtt("dsp_ing_czcs_filmfr", D_LONG, 1, &b->filmfr);
    GlbAtt("dsp_ing_czcs_year", D_SHORT, 1, &b->year);
    GlbAtt("dsp_ing_czcs_daysmp", D_SHORT, 1, &b->daysmp);
    GlbAtt("dsp_ing_czcs_milsec", D_LONG, 1, &b->milsec);
    GlbAtt("dsp_ing_czcs_decom1", D_LONG, 1, &b->decom1);
    GlbAtt("dsp_ing_czcs_decom2", D_LONG, 1, &b->decom2);
    GlbAtt("dsp_ing_czcs_confi1", D_CHAR, 1, &b->confi1);
    GlbAtt("dsp_ing_czcs_confi2", D_CHAR, 1, &b->confi2);
    GlbAtt("dsp_ing_czcs_algrid", D_SHORT, 8, b->algrid);
    GlbAtt("dsp_ing_czcs_basplt", D_SHORT, 1, &b->basplt);
    GlbAtt("dsp_ing_czcs_slope", D_LONG, CMAXCHN, b->slope);
    GlbAtt("dsp_ing_czcs_intcpt", D_LONG, CMAXCHN, b->intcpt);
    GlbAtt("dsp_ing_czcs_tabadr", D_SHORT, 3, b->tabadr);
    GlbAtt("dsp_ing_czcs_cnlins", D_SHORT, 1, &b->cnlins);
    GlbAtt("dsp_ing_czcs_cncols", D_SHORT, 1, &b->cncols);
    GlbAtt("dsp_ing_czcs_czncs", D_SHORT, 1, &b->czncs);
    GlbAtt("dsp_ing_czcs_line", D_SHORT, 1, &b->line);
    GlbAtt("dsp_ing_czcs_icol", D_SHORT, 1, &b->icol);
    GlbAtt("dsp_ing_czcs_date0", D_CHAR, 10, b->date0);
    GlbAtt("dsp_ing_czcs_time0", D_CHAR, 8, b->time0);
    GlbAtt("dsp_ing_czcs_label", D_SHORT, 3, b->label);
    GlbAtt("dsp_ing_czcs_nxtblk", D_SHORT, 1, &b->nxtblk);
    GlbAtt("dsp_ing_czcs_datblk", D_SHORT, 1, &b->datblk);
    GlbAtt("dsp_ing_czcs_itape", D_SHORT, 1, &b->itape);
    GlbAtt("dsp_ing_czcs_cbias", D_SHORT, 1, &b->cbias);
    GlbAtt("dsp_ing_czcs_ccoeff", D_SHORT, 1, &b->ccoeff);
    GlbAtt("dsp_ing_czcs_pastim", D_LONG, 1, &b->pastim);
    GlbAtt("dsp_ing_czcs_passcn", D_SHORT, 1, &b->passcn);
    GlbAtt("dsp_ing_czcs_lostct", D_SHORT, 1, &b->lostct);
    GlbAtt("dsp_ing_czcs_lost", D_SHORT, 20, b->lost);
    GlbAtt("dsp_ing_czcs_ndrll", D_SHORT, 1, &b->ndrll);
    GlbAtt("dsp_ing_czcs_ndrrec", D_SHORT, CMAXNLL, b->ndrrec);
    GlbAtt("dsp_ing_czcs_ndrlat", D_SHORT, CMAXNLL, b->ndrlat);
    GlbAtt("dsp_ing_czcs_ndrlon", D_SHORT, CMAXNLL, b->ndrlon);
    GlbAtt("dsp_ing_czcs_chncnt", D_SHORT, 1, &b->chncnt);
    GlbAtt("dsp_ing_czcs_chndsq", D_SHORT, CMAXCHN, b->chndsq);
    GlbAtt("dsp_ing_czcs_czncs2", D_SHORT, 1, &b->czncs2);
    GlbAtt("dsp_ing_czcs_corbit", D_SHORT, 1, &b->corbit);
    GlbAtt("dsp_ing_czcs_numscn", D_SHORT, 1, &b->numscn);
    GlbAtt("dsp_ing_czcs_rpmclc", D_FLOAT, 1, &b->rpmclc);
    GlbAtt("dsp_ing_czcs_slope2", D_LONG, CMAXCHN, b->slope2);
    GlbAtt("dsp_ing_czcs_intcp2", D_LONG, CMAXCHN, b->intcp2);
    GlbAtt("dsp_ing_czcs_tiltd", D_FLOAT, 1, &b->tiltd);
    GlbAtt("dsp_ing_czcs_tiltr", D_FLOAT, 1, &b->tiltr);
    GlbAtt("dsp_ing_czcs_czvers", D_SHORT, 1, &b->czvers);
    GlbAtt("dsp_ing_czcs_cztmrv", D_SHORT, 1, &b->cztmrv);
    GlbAtt("dsp_ing_czcs_czcpgm", D_SHORT, 1, &b->czcpgm);
    GlbAtt("dsp_ing_czcs_paserr", D_LONG, 1, &b->paserr);
    GlbAtt("dsp_ing_czcs_zpitch", D_FLOAT, 1, &b->zpitch);
    GlbAtt("dsp_ing_czcs_zroll", D_FLOAT, 1, &b->zroll);
    GlbAtt("dsp_ing_czcs_zyaw", D_FLOAT, 1, &b->zyaw);

    return status;
}


/*
 *	Convert a TIROS ingest-block.
 */
int
ConvertTiros(TIRO_INGEST_HDR * b)
{
    int status = 1;

    GlbAtt("dsp_ing_tiros_ourid", D_CHAR, 8, b->ourid);
    GlbAtt("dsp_ing_tiros_numscn", D_SHORT, 1, &b->numscn);
    GlbAtt("dsp_ing_tiros_idsat", D_SHORT, 1, &b->idsat);
    GlbAtt("dsp_ing_tiros_iddata", D_SHORT, 1, &b->iddata);
    GlbAtt("dsp_ing_tiros_year", D_SHORT, 1, &b->year);
    GlbAtt("dsp_ing_tiros_daysmp", D_SHORT, 1, &b->daysmp);
    GlbAtt("dsp_ing_tiros_milsec", D_LONG, 1, &b->milsec);
    GlbAtt("dsp_ing_tiros_slope", D_LONG, MAXCHN, b->slope);
    GlbAtt("dsp_ing_tiros_intcpt", D_LONG, MAXCHN, b->intcpt);
    GlbAtt("dsp_ing_tiros_tabadr", D_SHORT, 3, b->tabadr);
    GlbAtt("dsp_ing_tiros_cnlins", D_SHORT, 1, &b->cnlins);
    GlbAtt("dsp_ing_tiros_cncols", D_SHORT, 1, &b->cncols);
    GlbAtt("dsp_ing_tiros_czncs", D_SHORT, 1, &b->czncs);
    GlbAtt("dsp_ing_tiros_line", D_SHORT, 1, &b->line);
    GlbAtt("dsp_ing_tiros_icol", D_SHORT, 1, &b->icol);
    GlbAtt("dsp_ing_tiros_date0", D_CHAR, 10, b->date0);
    GlbAtt("dsp_ing_tiros_time0", D_CHAR, 8, b->time0);
    GlbAtt("dsp_ing_tiros_label", D_SHORT, 3, b->label);
    GlbAtt("dsp_ing_tiros_nxtblk", D_SHORT, 1, &b->nxtblk);
    GlbAtt("dsp_ing_tiros_datblk", D_SHORT, 1, &b->datblk);
    GlbAtt("dsp_ing_tiros_itape", D_SHORT, 1, &b->itape);
    GlbAtt("dsp_ing_tiros_cbias", D_SHORT, 1, &b->cbias);
    GlbAtt("dsp_ing_tiros_ccoeff", D_SHORT, 1, &b->ccoeff);
    GlbAtt("dsp_ing_tiros_pastim", D_LONG, 1, &b->pastim);
    GlbAtt("dsp_ing_tiros_passcn", D_SHORT, 1, &b->passcn);
    GlbAtt("dsp_ing_tiros_lostct", D_SHORT, 1, &b->lostct);
    GlbAtt("dsp_ing_tiros_lost", D_SHORT, 20, b->lost);
    GlbAtt("dsp_ing_tiros_ndrll", D_SHORT, 1, &b->ndrll);
    GlbAtt("dsp_ing_tiros_ndrrec", D_SHORT, MAXNLL, b->ndrrec);
    GlbAtt("dsp_ing_tiros_ndrlat", D_SHORT, MAXNLL, b->ndrlat);
    GlbAtt("dsp_ing_tiros_ndrlon", D_SHORT, MAXNLL, b->ndrlon);
    GlbAtt("dsp_ing_tiros_chncnt", D_SHORT, 1, &b->chncnt);
    GlbAtt("dsp_ing_tiros_chndsq", D_SHORT, MAXCHN, b->chndsq);
    GlbAtt("dsp_ing_tiros_czncs2", D_SHORT, 1, &b->czncs2);
    GlbAtt("dsp_ing_tiros_wrdsiz", D_SHORT, 1, &b->wrdsiz);
    GlbAtt("dsp_ing_tiros_nchbas", D_SHORT, 1, &b->nchbas);
    GlbAtt("dsp_ing_tiros_nchlst", D_SHORT, 1, &b->nchlst);
    GlbAtt("dsp_ing_tiros_rpmclc", D_FLOAT, 1, &b->rpmclc);
    GlbAtt("dsp_ing_tiros_numpix", D_SHORT, 1, &b->numpix);
    GlbAtt("dsp_ing_tiros_scnden", D_SHORT, 1, &b->scnden);
    GlbAtt("dsp_ing_tiros_eltden", D_SHORT, 1, &b->eltden);
    GlbAtt("dsp_ing_tiros_orbtno", D_SHORT, 1, &b->orbtno);
    GlbAtt("dsp_ing_tiros_slope2", D_LONG, MAXCHN, b->slope2);
    GlbAtt("dsp_ing_tiros_intcp2", D_LONG, MAXCHN, b->intcp2);
    GlbAtt("dsp_ing_tiros_prtemp", D_FLOAT, 1, &b->prtemp);
    GlbAtt("dsp_ing_tiros_timerr", D_FLOAT, 1, &b->timerr);
    GlbAtt("dsp_ing_tiros_timstn", D_SHORT, 1, &b->timstn);

    return status;
}


/*
 *	Convert a CZCS sub-commutation block.
 */
int
ConvertCzcsSubcom(ASSOC_ENTRY * ae)
{
    int nbytes = ae->AssocTotalSize;	/* amount to read */
    int block = ae->AssocBlockPtr;	/* which block */
    CZCS_SUBCOM subcom;		/* CZCS subcom  */
    static char me[] = "dsp server: ConvertCzcsSubcom,";	/* this routine's name */

    if (!ReadBytes
	(&InImage->FileDescriptor, (char *) &subcom, &block, &nbytes)) {
	ErrorExit(me, "Couldn't read CZCS sub-commutation block");
    }
    else {
	GlbAtt("dsp_subcom_mean", D_FLOAT,
	       sizeof(subcom.dummy) / sizeof(float),
	       (char *) subcom.dummy);
	GlbAtt("dsp_subcom_nobs", D_LONG,
	       sizeof(subcom.idmcnt) / sizeof(Dlong),
	       (char *) subcom.idmcnt);
    }

    return 1;
}


/*
 *	Convert database control block.
 */
int
ConvertDbControl(ASSOC_ENTRY * ae)
{
    char dbRecord[DSP_BLOCKSIZE];
    int block = ae->AssocBlockPtr;
    int nbytes = ae->AssocTotalSize;
    static char me[] = "dsp server: ConvertDbControl,";

    if (!ReadBytes(&InImage->FileDescriptor, dbRecord, &block, &nbytes)) {
	ErrorExit(me, "Couldn't read database-control block");
    }
    else {
	GlbAtt("dsp_dbcontrol", D_LONG, nbytes / sizeof(int),
	       (char *) dbRecord);
    }

    return 1;
}


/*
 *	Convert a palette.
 */
int
ConvertPalette(ASSOC_ENTRY * ae)
{
    int block = ae->AssocBlockPtr;
    int nbytes = ae->AssocTotalSize;
    int PalNo = ae->AssocDataType - ASS_PALETTE_1 + 1;
    char name[DSP_BLOCKSIZE];
    PALETTE pal;
    static char me[] = "dsp server: ConvertPalette,";

    if (!ReadBytes
	(&InImage->FileDescriptor, (char *) pal, &block, &nbytes)) {
	ErrorExit(me, "Couldn't read palette #%d block", PalNo);
    }
    else {
	sprintf(name, "dsp_palette_%d", PalNo);
	GlbAtt(name, D_SHORT, nbytes / sizeof(short), (char *) pal);
    }

    return 1;
}


/*
 *	Convert audit records.
 */
int
ConvertAuditRecords(ASSOC_ENTRY * ae)
{
    int status = 1;
    int AuditSize = ae->AssocTotalSize;
    /* length of DSP audit records.  */
    /* NB: includes terminating '\0' */
    char *history;		/* history buffer */
    static char me[] = "dsp server: ConvertAuditRecords,";

    DBGc(me, "Allocating buffer");
    if ((history = malloc((unsigned) AuditSize + 512)) == NULL) {
	ErrorExit(me, "Couldn't allocate history-buffer");
	status = 0;
    }
    else {
	if (AuditSize > 0) {	/* if audit records exist */
	    int DataType = ASS_AUDIT;
	    int AssocFlag;
	    char AssocAnnot[MAX_ASSOC_ANNOT];	/* Annot field for this data */

	    DBGc(me, "Getting audit records");

	    if (!Dsp_GetAssocBlock(&InImage, history, &AuditSize,
				   &DataType, &AssocFlag, AssocAnnot)) {
		ErrorExit(me, "Couldn't get audit block");
		status = 0;
	    }
	    else {
		AuditSize = strlen(history) + 1;
		history[AuditSize - 1] = '\n';
	    }
	}
	if (status) {
	    DBGc(me, "Setting history attribute");
	    GlbStrAtt("history", history);
	}
    }
    (void) free(history);

    return status;
}

// $Log: DspDodsUtilC.c,v $
// Revision 1.6  2004/07/09 17:41:01  jimg
// Merged with release-3-4-3FCS.
//
// Revision 1.5.4.1  2004/04/13 20:28:43  jimg
// Fix for deprecated use of varargs.h. I've switched to the new (1990!) stdarg
// types. This addresses a problem that shows up with gcc 3.3 which no longer
// supports the deprecated varargs header.
//
// Revision 1.5  2003/02/10 21:19:02  jimg
// Merged changes from 3.2.4.
//
// Revision 1.4.26.1  2001/11/27 20:47:07  jimg
// I changed the DSP include files from "..." to <...> so that the
// dependencies would not include these files. This made the Makefile.in
// too fragile.
//
// Revision 1.4  1998/02/05 20:14:20  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.3  1997/03/10 05:41:49  reza
// Added error objects to the code and updated for release 2.12.
// Now the data is sent gzip compressed.
//
// Revision 1.2  1996/11/15 18:22:27  jimg
// Fixed copyright statement.
// Added include of config header
//
// Revision 1.1  1996/09/18 22:42:16  reza
//  First release of DODS-DSP servers.
