
// (c) COPYRIGHT URI/MIT 1996, 1997
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//	reza		Reza Nekovei (reza@intcomm.net)
//

#include "config_dsp.h"

static char id[] not_used ={"$Id: DspDodsUtilCc.cc 13340 2006-03-03 16:10:13Z jimg $"};

#include <string>

#include "dods-datatypes.h"
#include "AttrTable.h"
#include "DAS.h"
#include "DDS.h"
#include "util.h"
#include "cgi_util.h"

#include "DSPStr.h"
#include "DSPByte.h"
#include "DSPInt32.h"
#include "DSPFloat64.h"
#include "DSPArray.h"
#include "DSPGrid.h"
#include "Dtype.h"

extern "C" int OpenDsp(const char *filename);
extern "C" int DefineDDS();
extern "C" int DefineDAS();

// These are used as the return values for print_type().

static const char STRING[]="String";
static const char BYTE[]="Byte";
static const char INT32[]="Int32";
static const char FLOAT64[]="Float64";

// Used by ErrMsgT

static char Msgt[255];

static DDS *dds_table;
static DAS *das_table;

extern "C" void
SendError(ErrorCode ec, char *msg)
{
    string errMsg;
    errMsg = (string)"\"" + msg + "\"";
    Error *ErrorObj = new Error(ec, errMsg);
    set_mime_text(stdout, dods_error);
    ErrorObj->print(stdout);
}


// remove quotes, DAS parser does not like extra "quotes"
void
strclean(char *ValBuf, char *vals, int ChrLen)
{
    for (int m=0; m < ChrLen; m++)
	if( *((char *)vals+m) != '"')
	    *(ValBuf++) = * ((char *)vals+m);
}

// Given the type, array number and pointer to a attribute, build the printed
// representation of that attribute. 
// Return a char * to newly allocated memory. The caller must call delete []

static char *
print_attr(D_type type, int loc, void *vals, int ChrLen)
{
    char *rep;                  // return value
    char *ValBuf;
    union {
	char *cp;
	short *sp;
	Dlong *lp;
	float *fp;
	double *dp;
    } gp;

    switch (type) {
      case D_BYTE:
	//      unsigned char uc;
	char uc;
	rep = new char [32];
	gp.cp = (char *) vals;
	uc = *(gp.cp+loc);
	sprintf (rep, "%d",uc);
	return rep;
	break;

      case D_CHAR:
	if(ChrLen == 0) // default length
	    ChrLen = strlen((const char *)vals);
	rep = new char [ChrLen + 3];
	ValBuf = new char [ChrLen + 1];
	strclean(ValBuf, (char *)vals, ChrLen);
	*(ValBuf + ChrLen) = '\0';	       
	sprintf(rep, "\"%s\"", (const char *)ValBuf);
	delete [] ValBuf;
	return rep;
	break;

      case D_SHORT:
	rep = new char [32];
	gp.sp = (short *) vals;
	sprintf (rep, "%d",*(gp.sp+loc));
	return rep;
	break;
       
      case D_LONG:
	rep = new char [32];
	gp.lp = (Dlong *) vals; 
	sprintf (rep, LD ,*(gp.lp+loc));
	return rep;
	break;
       
      case D_FLOAT: {
	  char *f_fmt = "%.8g";
	  char gps[30];           // for ascii of a float and double precision
	  rep = new char [32];
	  gp.fp = (float *) vals;
	  int ll;
	  (void) sprintf(gps, f_fmt, * (gp.fp+loc));
	  // make sure we get a decimal point for float type attributes
	  ll = strlen(gps);
	  if (!strchr(gps, '.') && !strchr(gps,'e')) 
	      gps[ll++] = '.';
	  gps[ll] = '\0';
	  sprintf (rep, "%s", gps);
	  return rep;
	  break;
      }
    
      case D_DOUBLE: {
	  char *d_fmt = "%.16g";
	  char gps[30];           // for ascii of a float and double precision

	  rep = new char [32];
	  gp.dp = (double *) vals;
	  (void) sprintf(gps, d_fmt, *(gp.dp+loc));
	  // make sure we get a decimal point for float type attributes
	  int ll = strlen(gps);
	  if (!strchr(gps, '.') && !strchr(gps,'e')) 
	      gps[ll++] = '.';
	  gps[ll] = '\0';
	  sprintf (rep, "%s", gps);
	  return rep;
	  break;
      }
      default:
	ErrMsgT("dsp_das: print_attr: bad data type");
	return NULL;
    }
}


// Return the printed representation of a netcdf type -- in a form that dods
// can use.
//
// Returns: a char * referencing the datatype's printed representation. The
// char * is static.

static const char *
print_type(D_type datatype) 
{
    switch (datatype) {
      case D_CHAR:
	return STRING;

      case D_BYTE:
	return BYTE;

      case D_SHORT:
      case D_LONG:
	return INT32;

      case D_FLOAT:
      case D_DOUBLE:
	return FLOAT64;

      default:
	return STRING;
    }
}

// Add an attribute table.
extern "C" int
AddAttTab(char *varname, register AttEntry_t *tab)
{
    AttrTable	        *attr_table;

    attr_table = das_table->get_table(varname);
    if (!attr_table)
	attr_table = das_table->add_table(varname, new AttrTable);
    
    for (; tab->name != NULL; ++tab){
	int len = 0;    
	int size = tab->size;
	bool empty = false;
	if (tab->type == D_CHAR){ // One string
	    len = size;
	    size = 1;
	    if(strlen(tab->value) == 0)
		empty = true;
	}
	if(!empty)// don't add empty string attributes
	    for (int loc=0; loc < size ; loc++) {
		char *print_rep = print_attr(tab->type, loc, tab->value, len);
		if (attr_table->append_attr(tab->name, print_type(tab->type),
					    print_rep) == 0){
		    sprintf (Msgt,"DSP server: Couldn't output attribute \"%s\"", tab->name);
		    ErrMsgT(Msgt);	  
		    SendError(unknown_error, Msgt);
		    exit(1);
		}
		delete [] print_rep;
	    }
    }
       
    if(tab->name == NULL)
	return 1;

    return 0;
}

// Add an attribute
extern "C" void
AddAtt(char *varname, char *attname, D_type type, int size, void *value)
{
    AttrTable *attr_table;
    int       Len = 0;
    bool empty = false;

    attr_table = das_table->get_table(varname);
    if (!attr_table)
	attr_table = das_table->add_table(varname, new AttrTable);

    if (type == D_CHAR) {
	Len =size; // string length
	size = 1; // one string
	if(strlen((char *)value) == 0)
	    empty = true;
    }
    if(!empty) // don't add empty string attributes
	for (int loc=0; loc < size ; loc++) {
	    char *print_rep = print_attr(type, loc, value, Len);
	    if (attr_table->append_attr(attname, print_type(type), print_rep) == 0){
		sprintf (Msgt,"dsp_das server: couldn't output attribute \"%s\"", attname); 
		ErrMsgT(Msgt);
		SendError(unknown_error, Msgt);
		exit(1);
	    }
	    delete [] print_rep;
	} 
}

extern "C" void
StrAtt(char *varname, char *attname, void *value)
{
    AddAtt(varname, attname, D_CHAR, 0, value);
}

extern "C" void
GlbAtt(char *attname,D_type type, int size, void *value)
{
    AddAtt("NC_GLOBAL", attname, type, size, value);
}

extern "C" void
GlbStrAtt(char *attname, void *value)
{
    AddAtt("NC_GLOBAL", attname, D_CHAR, 0, value);
}


// This function returns the appropriate DODS BaseType.
//
BaseType *
Get_bt(char *varname, D_type datatype) 
{
    switch (datatype) {
      case D_CHAR:
        return new DSPStr(varname);

      case D_BYTE:
        return new DSPByte(varname);

      case D_SHORT:
      case D_LONG:
        return new DSPInt32(varname);

      case D_FLOAT:
      case D_DOUBLE:
        return new DSPFloat64(varname);

      default:
        return new DSPStr(varname);
    }
}

// Make a general image array
extern "C" int
MakeArray(char * VarName, D_type VarType, int DimSiz[2])
{  
    Array *ar;
    BaseType *bt = Get_bt(VarName,VarType);

    ar = new DSPArray(VarName);
    ar->add_var(bt);
    ar->append_dim(DimSiz[0],"line");
    ar->append_dim(DimSiz[1],"pixel");
    dds_table->add_var(ar);
  
    return 1;
}



// Make a gridded image (know lat and lon maps)
extern "C" int
MakeGrid(char * VarName, D_type VarType, int DimSiz[2])
{
    Array *ar; // pointers
    Array *lat; 
    Array *lon; 
    Grid *gr;
    Part pr;
    BaseType *bt;
    // Support for direct access to map vectors of clients without grid type

    lat = new DSPArray; 
    bt = Get_bt("lat",D_FLOAT);
    lat->add_var(bt);     
    lat->append_dim(DimSiz[0],"lat");

    lon = new DSPArray; 
    bt = Get_bt("lon",D_FLOAT);
    lon->add_var(bt);     
    lon->append_dim(DimSiz[1],"lon");

    dds_table->add_var(lat); 
    dds_table->add_var(lon); 
  
    // Now build the regular Grid.

    bt = Get_bt(VarName,VarType);

    ar = new DSPArray(VarName);
    ar->add_var(bt);
    ar->append_dim(DimSiz[0],"lat");
    ar->append_dim(DimSiz[1],"lon");

    gr = new DSPGrid(VarName);
    pr = array; 
    gr->add_var(ar,pr); // create array part of the grid
  
    pr = maps; 
  
    ar = new DSPArray; 
    bt = Get_bt("lat",D_FLOAT);
    ar->add_var(bt);     
    ar->append_dim(DimSiz[0],"lat");
    gr->add_var(ar,pr); // create the first map vector for the grid

    ar = new DSPArray; 
    bt = Get_bt("lon",D_FLOAT);
    ar->add_var(bt);     
    ar->append_dim(DimSiz[1],"lon");
    gr->add_var(ar,pr); // create the second map vector for the grid

    dds_table->add_var(gr); // add the grid to DDS table

    return 1;									   
}


// Create and return the DDS for the DSP image file
bool
read_descriptors(DDS &dds, const string &filename, string *err_msg)
{
#if 0
    if (dds_table)
	delete dds_table;

    dds_table = new DDS();
#endif

    dds_table = &dds;		// Static global used by extern "C" code
				// above. 
    if (!OpenDsp(filename.c_str())) {
	sprintf (Msgt,"dspdds server: Could not open file %s ", filename.c_str());
	ErrMsgT(Msgt);
	// Let the fitler send the error back to the client. jhrg 12/30/97
	// SendError(no_such_file, Msgt);
	*(err_msg) = (string)"\"" + Msgt + "\"";
	return false;
    }
    
    // dataset name
    dds.set_dataset_name(name_path(filename));
    
    // read variables class
    if (!DefineDDS())
	return false;
 
    return true;
}

// Create and return the DAS for the DSP image file
bool
read_attributes(DAS &das, const string &filename, string *err_msg)
{
    das_table = &das;		// Static global; same as dds_table above.

    if (!OpenDsp(filename.c_str())) {
	sprintf (Msgt,"dspdas server: could not open file %s ", filename.c_str());
	ErrMsgT(Msgt);
	//SendError(unknown_error, Msgt);
	*(err_msg) = (string)"\"" + Msgt + "\"";
	return false;
    }
    
    // read variables class
    if (!DefineDAS())
	return false;
  
    return true;
}

// $Log: DspDodsUtilCc.cc,v $
// Revision 1.12  2003/02/10 21:19:02  jimg
// Merged changes from 3.2.4.
//
// Revision 1.11.4.2  2002/06/28 16:44:51  jimg
// Changes to the Connect class changed the set of headers included by other
// headers; I fixed the includes of these files to compensate.
//
// Revision 1.11.4.1  2002/01/30 20:33:23  jimg
// Constraints are failing when applied to the extra map vectors that are
// created at the top level of the DDS. I think it's because the code in the
// core is constraining the Grid's map vectors (which have the same names) but
// yet marking the extra map vectors. So when the data are returned, the
// constraint does not get applied and by default the whole vector is sent. A
// fix for this is to move the extra vectors so that they appear at the top of
// the DDS, not at the end.
//
// Revision 1.11  2000/10/09 21:24:59  jimg
// Moved CVS Log entries to the end of each file.
// Changed the definition of the read method to match the dap library.
// Added exceptions to the read methods.
// Added code to catch exceptions in dsp_das, _dds and _dods.
//
// Revision 1.10  1999/04/30 21:32:47  jimg
// Merged no-gnu and release-2-17
//
// Revision 1.9.8.1  1999/02/10 06:28:58  brent
// convert to Standard C++ strings -- b^2
//
// Revision 1.9  1998/03/20 00:29:51  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.8  1998/02/05 20:14:20  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.7  1997/10/09 22:19:34  jimg
// Resolved conflicts in merge of 2.14c to trunk.
//
// Revision 1.6  1997/08/06 01:26:58  jimg
// Added calls to read_descriptors and read_attributes that create new
// instances of the global pointers dds_table and das_table.
//
// Revision 1.5  1997/07/15 17:53:36  jimg
// merged release-2-1-4 changes
//
// Revision 1.4.2.1  1997/07/15 00:53:24  jimg
// Fixed as per Dan's changes for version 2.14; changed the DAS and DDS objects
// so that the are pointers and not global static objects.
//
