
# m4 macros from the Unidata netcdf 2.3.2 pl4 distribution. Modified for use
# with GNU Autoconf 2.1. I renamed these from UC_* to DODS_* so that there
# will not be confusion when porting future versions of netcdf into the DODS
# source distribution. Unidata, Inc. wrote the text of these macros and holds
# a copyright on them.
#
# jhrg 3/27/95
#
# Hacked many times. No much reduced in size. Virtually non of the Unidata
# macros remain in this file. jhrg 5/09/05
#
# $Id: acinclude.m4 11963 2005-08-17 19:39:57Z jimg $

AC_DEFUN([DODS_CHECK_SIZES], [dnl
    # Ignore the errors about AC_TRY_RUN missing an argument. jhrg 5/2/95

    AC_REQUIRE([AC_PROG_CC])

    AC_CHECK_SIZEOF(int)
    AC_CHECK_SIZEOF(long)
    AC_CHECK_SIZEOF(char)
    AC_CHECK_SIZEOF(double)
    AC_CHECK_SIZEOF(float)

    # Now generate symbols that define the dods_int32, ..., types
    # based on this machine's notion of an int, etc. See dods-datatypes.h.in.
    # I've separated the typedefs from the config_dap.h header because other
    # projects which use the DAP were getting conflicts with their includes,
    # or the includes of still other libraries, and config_dap.h. The 
    # config_dap.h header is now included only by .cc and .c files and headers
    # that need the typedefs use dods-datatypes.h. The code below makes 
    # dods-datatypes.h stand on its own. 8/2/2000 jhrg

    if test $ac_cv_sizeof_long -eq 4 
    then
	DODS_INT32=long
	DODS_UINT32="unsigned long"
	XDR_INT32=xdr_long
	XDR_UINT32=xdr_u_long
    elif test $ac_cv_sizeof_int -eq 4
    then
	DODS_INT32=int
	DODS_UINT32="unsigned int"
	XDR_INT32=xdr_int
	XDR_UINT32=xdr_u_int
    else
	AC_MSG_ERROR(How do I get a 32 bit integer on this machine?)
    fi
    # I'm using the three arg form of AC_DEFINE_UNQUOTED because autoheader
    # needs the third argument (although I don't quite get the specifics... 
    # 2/15/2001 jhrg
    AC_DEFINE_UNQUOTED(DINT32, $DODS_INT32, [int32])
    AC_DEFINE_UNQUOTED(DUINT32, $DODS_UINT32, [uint32])
    AC_DEFINE_UNQUOTED(XDR_INT32, $XDR_INT32, [xdr int32])
    AC_DEFINE_UNQUOTED(XDR_UINT32, $XDR_UINT32, [xdr uint32])

    # Assume nobody's hosed the 16-bit types...
    DODS_INT16=short
    DODS_UINT16="unsigned short"
    XDR_INT16=xdr_short
    XDR_UINT16=xdr_u_short
    AC_DEFINE_UNQUOTED(DINT16, $DODS_INT16, [dint16])
    AC_DEFINE_UNQUOTED(DUINT16, $DODS_UINT16, [uint16])
    AC_DEFINE_UNQUOTED(XDR_INT16, $XDR_INT16, [xdr int16])
    AC_DEFINE_UNQUOTED(XDR_UINT16, $XDR_UINT16, [xdr uint16])

    if test $ac_cv_sizeof_char -eq 1
    then
	DODS_BYTE="unsigned char"
    else
	AC_MSG_ERROR(How do I get an 8 bit unsigned integer on this machine?)
    fi
    AC_DEFINE_UNQUOTED(DBYTE, $DODS_BYTE, [dbyte])

    if test $ac_cv_sizeof_double -eq 8
    then
	DODS_FLOAT64=double
	DODS_FLOAT32=float
	XDR_FLOAT64=xdr_double
	XDR_FLOAT32=xdr_float
    else
	AC_MSG_ERROR(How do I get floating point types on this machine?)
    fi
    AC_DEFINE_UNQUOTED(DFLOAT64, $DODS_FLOAT64, [dfloat64])
    AC_DEFINE_UNQUOTED(DFLOAT32, $DODS_FLOAT32, [dfloat32])
    AC_DEFINE_UNQUOTED(XDR_FLOAT64, $XDR_FLOAT64, [xdr float64])
    AC_DEFINE_UNQUOTED(XDR_FLOAT32, $XDR_FLOAT32, [xdr float32])])

AC_DEFUN([OPENDAP_DEBUG_OPTION], [dnl
    AC_ARG_ENABLE(debug, 
		  [  --enable-debug=ARG      Program instrumentation (1,2)],
		  DEBUG=$enableval, DEBUG=no)

    case "$DEBUG" in
    no) 
      ;;
    1)
      AC_MSG_RESULT(Setting debugging to level 1)
      AC_DEFINE(DODS_DEBUG)
      ;;
    2) 
      AC_MSG_RESULT(Setting debugging to level 2)
      AC_DEFINE(DODS_DEBUG, , [Set instrumentation to level 1 (see debug.h)])
      AC_DEFINE(DODS_DEBUG2, , [Set instrumentation to level 2])
      ;;
    *)
      AC_MSG_ERROR(Bad debug value)
      ;;
    esac])

# cross-compile problem with test option -d
AC_DEFUN(OPENDAP_DSP_ROOT, [dnl

    AC_ARG_WITH(dsp,
		[  --with-dsp=DIR          Directory containing DSP software from U of Miami],
		DSP_ROOT=${withval}, DSP_ROOT="$DSP_ROOT")

    if test ! -d "$DSP_ROOT"
    then
        DSP_ROOT=""
    fi
    if test -z "$DSP_ROOT"
    then
	AC_MSG_CHECKING(for the DSP library root directory)

	for p in /usr/local/src/DSP /usr/local/DSP \
		 /usr/local/src/dsp /usr/local/dsp \
		 /usr/contrib/src/dsp /usr/contrib/dsp \
		 /usr/dsp /data1/dsp
	do
	    if test -z "$DSP_ROOT"
	    then
	    	for d in `ls -dr ${p}* 2>/dev/null`
		do
		    if test -f ${d}/inc/dsplib.h
		    then
		        DSP_ROOT=${d}
		        break
		    fi
	        done
	    fi
	done
    fi

    if test "$DSP_ROOT"
    then
	AC_SUBST(DSP_ROOT)
	dnl Only add this path to gcc's options... jhrg 11/15/96
	CPPFLAGS="$CPPFLAGS -I\$(DSP_ROOT)/inc"
	LDFLAGS="$LDFLAGS -L\$(DSP_ROOT)/lib -L\$(DSP_ROOT)/shlib"
	AC_MSG_RESULT(Set DSP root directory to $DSP_ROOT) 
    else
        AC_MSG_ERROR(not found! see configure --help)
    fi])


