
// -*- mode: c++; c-basic-offset:4 -*-

// This file is part of libdap, A C++ implementation of the OPeNDAP Data
// Access Protocol.

// Copyright (c) 2002,2003 OPeNDAP, Inc.
// Author: James Gallagher <jgallagher@opendap.org>
//         Dan Holloway <dan@hollywood.gso.uri.edu>
//         Reza Nekovei <reza@intcomm.net>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.

#include "config_dsp.h"

static char *rcsid not_used =
    "$Id: dsp_handler.cc 13716 2006-04-19 16:56:05Z jimg $";

#include <iostream>
#include <string>

#include "DODSFilter.h"
#include "DDS.h"
#include "DAS.h"
#include "DataDDS.h"
#include "ConstraintEvaluator.h"

#include "ObjectType.h"
#include "cgi_util.h"

#include "DSPTypeFactory.h"

extern bool read_descriptors(DDS & dds, const string & filename,
                             string * error);
extern bool read_attributes(DAS & das, const string & filename,
                            string * error);

const string cgi_version = DODS_SERVER_VERSION;

int main(int argc, char *argv[])
{
    try {
        DODSFilter df(argc, argv);

        switch (df.get_response()) {
        case DODSFilter::DAS_Response:{
                DAS das;

                string err_msg;
                if (!read_attributes(das, df.get_dataset_name(), &err_msg))
                    throw Error(err_msg);

                df.read_ancillary_das(das);
                df.send_das(das);
                break;
            }

        case DODSFilter::DDS_Response:{
                DSPTypeFactory dfactory;
                DDS dds(&dfactory);
                ConstraintEvaluator ce;

                dds.filename(df.get_dataset_name());

                string err_msg;
                if (!read_descriptors
                    (dds, df.get_dataset_name(), &err_msg))
                    throw Error(err_msg);

                df.read_ancillary_dds(dds);
                df.send_dds(dds, ce, true);
                break;
            }

        case DODSFilter::DataDDS_Response:{
                DSPTypeFactory dfactory;
                DDS dds(&dfactory);
                ConstraintEvaluator ce;

                dds.filename(df.get_dataset_name());

                string err_msg;
                if (!read_descriptors
                    (dds, df.get_dataset_name(), &err_msg))
                    throw Error(err_msg);

                df.read_ancillary_dds(dds);
                df.send_data(dds, ce, stdout);
                break;
            }

        case DODSFilter::DDX_Response:{
                DSPTypeFactory dfactory;
                DDS dds(&dfactory);
                DAS das;
                ConstraintEvaluator ce;

                dds.filename(df.get_dataset_name());

                string err_msg;
                if (!read_descriptors(dds, df.get_dataset_name(), &err_msg))
                    throw Error(err_msg);

                df.read_ancillary_dds(dds);

                if (!read_attributes(das, df.get_dataset_name(), &err_msg))
                    throw Error(err_msg);

                df.read_ancillary_das(das);

                dds.transfer_attributes(&das);

                df.send_ddx(dds, ce, stdout);
                break;
            }
        case DODSFilter::Version_Response:{
                if (df.get_cgi_version() == "")
                    df.set_cgi_version(cgi_version);
                df.send_version_info();

                break;
            }

        default:
            df.print_usage();   // Throws Error
        }
    }
    catch(Error & e) {
        set_mime_text(stdout, dods_error, cgi_version);
        e.print(stdout);
        return 1;
    }

    return 0;
}
